@@ -17,6 +17,7 @@
 
 #include "atticaprovider_p.h"
 
+#include "commentsmodel.h"
 #include "question.h"
 #include "tagsfilterchecker.h"
 
@@ -42,13 +43,15 @@ AtticaProvider::AtticaProvider(const QStringList &categories)
     , mInitialized(false)
 {
     // init categories map with invalid categories
-    foreach (const QString &category, categories) {
+    for (const QString &category : categories) {
         mCategoryMap.insert(category, Attica::Category());
     }
 
     connect(&m_providerManager, &ProviderManager::providerAdded, this, &AtticaProvider::providerLoaded);
     connect(&m_providerManager, SIGNAL(authenticationCredentialsMissing(Provider)),
             SLOT(authenticationCredentialsMissing(Provider)));
+    connect(this, &Provider::loadComments, this, &AtticaProvider::loadComments);
+    connect(this, &Provider::loadPerson, this, &AtticaProvider::loadPerson);
 }
 
 AtticaProvider::AtticaProvider(const Attica::Provider &provider, const QStringList &categories)
@@ -56,7 +59,7 @@ AtticaProvider::AtticaProvider(const Attica::Provider &provider, const QStringLi
     , mInitialized(false)
 {
     // init categories map with invalid categories
-    foreach (const QString &category, categories) {
+    for (const QString &category : categories) {
         mCategoryMap.insert(category, Attica::Category());
     }
     providerLoaded(provider);
@@ -122,10 +125,10 @@ void AtticaProvider::listOfCategoriesLoaded(Attica::BaseJob *listJob)
     qCDebug(KNEWSTUFFCORE) << "loading categories: " << mCategoryMap.keys();
 
     Attica::ListJob<Attica::Category> *job = static_cast<Attica::ListJob<Attica::Category>*>(listJob);
-    Category::List categoryList = job->itemList();
+    const Category::List categoryList = job->itemList();
 
     QList<CategoryMetadata> categoryMetadataList;
-    foreach (const Category &category, categoryList) {
+    for (const Category &category : categoryList) {
         if (mCategoryMap.contains(category.name())) {
             qCDebug(KNEWSTUFFCORE) << "Adding category: " << category.name() << category.displayName();
             //If there is only the placeholder category, replace it
@@ -209,7 +212,7 @@ void AtticaProvider::loadEntries(const KNSCore::Provider::SearchRequest &request
         categoriesToSearch = mCategoryMap.values();
     } else {
         categoriesToSearch.reserve(request.categories.size());
-        foreach (const QString &categoryName, request.categories) {
+        for (const QString &categoryName : qAsConst(request.categories)) {
             categoriesToSearch.append(mCategoryMap.values(categoryName));
         }
     }
@@ -223,7 +226,7 @@ void AtticaProvider::loadEntries(const KNSCore::Provider::SearchRequest &request
 
 void AtticaProvider::checkForUpdates()
 {
-    foreach (const EntryInternal &e, mCachedEntries) {
+    for (const EntryInternal &e : qAsConst(mCachedEntries)) {
         ItemJob<Content> *job = m_provider.requestContent(e.uniqueId());
         connect(job, &BaseJob::finished, this, &AtticaProvider::detailsLoaded);
         m_updateJobs.insert(job);
@@ -252,7 +255,7 @@ void AtticaProvider::detailsLoaded(BaseJob *job)
     if (m_updateJobs.remove(job) && m_updateJobs.isEmpty()) {
         qCDebug(KNEWSTUFFCORE) << "check update finished.";
         QList<EntryInternal> updatable;
-        foreach (const EntryInternal &entry, mCachedEntries) {
+        for (const EntryInternal &entry : qAsConst(mCachedEntries)) {
             if (entry.status() == KNS3::Entry::Updateable) {
                 updatable.append(entry);
             }
@@ -342,6 +345,79 @@ void AtticaProvider::loadPayloadLink(const KNSCore::EntryInternal &entry, int li
     }
 }
 
+void AtticaProvider::loadComments(const EntryInternal &entry, int commentsPerPage, int page)
+{
+    ListJob<Attica::Comment> *job = m_provider.requestComments(Attica::Comment::ContentComment, entry.uniqueId(), QLatin1String("0"), page, commentsPerPage);
+    connect(job, &BaseJob::finished, this, &AtticaProvider::loadedComments);
+    job->start();
+}
+
+/// TODO KF6 QList is discouraged, and we'll probably want to switch this (and the rest of the KNS library) to QVector instead
+QList<std::shared_ptr<KNSCore::Comment>> getCommentsList(const Attica::Comment::List &comments, std::shared_ptr<KNSCore::Comment> parent) {
+    QList<std::shared_ptr<KNSCore::Comment>> knsComments;
+    for (const Attica::Comment &comment : comments) {
+        qCDebug(KNEWSTUFFCORE) << "Appending comment with id" << comment.id() << ", which has" << comment.childCount() << "children";
+        auto knsComment = std::make_shared<KNSCore::Comment>();
+        knsComment->id = comment.id();
+        knsComment->subject = comment.subject();
+        knsComment->text = comment.text();
+        knsComment->childCount = comment.childCount();
+        knsComment->username = comment.user();
+        knsComment->date = comment.date();
+        knsComment->score = comment.score();
+        knsComment->parent = parent;
+        knsComments << knsComment;
+        if (comment.childCount() > 0) {
+            qCDebug(KNEWSTUFFCORE) << "Getting more comments, as this one has children, and we currently have this number of comments:" << knsComments.count();
+            knsComments << getCommentsList(comment.children(), knsComment);
+            qCDebug(KNEWSTUFFCORE) << "After getting the children, we now have the following number of comments:" << knsComments.count();
+        }
+    }
+    return knsComments;
+}
+
+void AtticaProvider::loadedComments(Attica::BaseJob *baseJob)
+{
+    if (!jobSuccess(baseJob)) {
+        return;
+    }
+
+    ListJob<Attica::Comment> *job = static_cast<ListJob<Attica::Comment>*>(baseJob);
+    Attica::Comment::List comments = job->itemList();
+
+    QList<std::shared_ptr<KNSCore::Comment>> receivedComments = getCommentsList(comments, nullptr);
+    emit commentsLoaded(receivedComments);
+}
+
+void AtticaProvider::loadPerson(const QString &username)
+{
+    if (m_provider.hasPersonService()) {
+        ItemJob<Attica::Person> *job = m_provider.requestPerson(username);
+        job->setProperty("username", username);
+        connect(job, &BaseJob::finished, this, &AtticaProvider::loadedPerson);
+        job->start();
+    }
+}
+
+void AtticaProvider::loadedPerson(Attica::BaseJob *baseJob)
+{
+    if (!jobSuccess(baseJob)) {
+        return;
+    }
+
+    ItemJob<Attica::Person> *job = static_cast<ItemJob<Attica::Person>*>(baseJob);
+    Attica::Person person = job->result();
+
+    auto author = std::make_shared<KNSCore::Author>();
+    author->setId(job->property("username").toString()); // This is a touch hack-like, but it ensures we actually have the data in case it is not returned by the server
+    author->setName(QString::fromLatin1("%1 %2").arg(person.firstName()).arg(person.lastName()).trimmed());
+    author->setHomepage(person.homepage());
+    author->setProfilepage(person.extendedAttribute(QStringLiteral("profilepage")));
+    author->setAvatarUrl(person.avatarUrl());
+    author->setDescription(person.extendedAttribute(QStringLiteral("description")));
+    emit personLoaded(author);
+}
+
 void AtticaProvider::accountBalanceLoaded(Attica::BaseJob *baseJob)
 {
     if (!jobSuccess(baseJob)) {
@@ -395,7 +471,7 @@ void AtticaProvider::downloadItemLoaded(BaseJob *baseJob)
 EntryInternal::List AtticaProvider::installedEntries() const
 {
     EntryInternal::List entries;
-    foreach (const EntryInternal &entry, mCachedEntries) {
+    for (const EntryInternal &entry : qAsConst(mCachedEntries)) {
         if (entry.status() == KNS3::Entry::Installed || entry.status() == KNS3::Entry::Updateable) {
             entries.append(entry);
         }
@@ -446,6 +522,8 @@ bool AtticaProvider::jobSuccess(Attica::BaseJob *job) const
     if (job->metadata().error() == Attica::Metadata::OcsError) {
         if (job->metadata().statusCode() == 200) {
             emit signalErrorCode(KNSCore::OcsError, i18n("Too many requests to server. Please try again in a few minutes."), job->metadata().statusCode());
+        } else if (job->metadata().statusCode() == 405) {
+            emit signalErrorCode(KNSCore::OcsError, i18n("The Open Collaboration Services instance %1 does not support the attempted function.").arg(name()), job->metadata().statusCode());
         } else {
             emit signalErrorCode(KNSCore::OcsError, i18n("Unknown Open Collaboration Service API error. (%1)", job->metadata().statusCode()), job->metadata().statusCode());
         }
@@ -462,6 +540,7 @@ EntryInternal AtticaProvider::entryFromAtticaContent(const Attica::Content &cont
     entry.setStatus(KNS3::Entry::Downloadable);
     entry.setVersion(content.version());
     entry.setReleaseDate(content.updated().date());
+    entry.setCategory(content.attribute(QStringLiteral("typeid")));
 
     int index = mCachedEntries.indexOf(entry);
     if (index >= 0) {
@@ -499,6 +578,7 @@ EntryInternal AtticaProvider::entryFromAtticaContent(const Attica::Content &cont
 
     entry.setLicense(content.license());
     Author author;
+    author.setId(content.author());
     author.setName(content.author());
     author.setHomepage(content.attribute(QStringLiteral("profilepage")));
     entry.setAuthor(author);
@@ -510,8 +590,8 @@ EntryInternal AtticaProvider::entryFromAtticaContent(const Attica::Content &cont
     entry.setTags(content.tags());
 
     entry.clearDownloadLinkInformation();
-    QList<Attica::DownloadDescription> descs = content.downloadUrlDescriptions();
-    foreach (const Attica::DownloadDescription &desc, descs) {
+    const QList<Attica::DownloadDescription> descs = content.downloadUrlDescriptions();
+    for (const Attica::DownloadDescription &desc : descs) {
         EntryInternal::DownloadLinkInformation info;
         info.name = desc.name();
         info.priceAmount = desc.priceAmount();
