@@ -22,6 +22,7 @@
 #include "engine.h"
 
 #include "../entry.h"
+#include "commentsmodel.h"
 #include "installation.h"
 #include "xmlloader.h"
 #include "imageloader_p.h"
@@ -63,6 +64,9 @@ public:
     Attica::ProviderManager *m_atticaProviderManager = nullptr;
     QStringList tagFilter;
     QStringList downloadTagFilter;
+    bool configLocationFallback = true;
+    QString name;
+    QMap<EntryInternal, CommentsModel*> commentsModels;
 };
 
 Engine::Engine(QObject *parent)
@@ -104,26 +108,47 @@ bool Engine::init(const QString &configfile)
 
     emit signalBusy(i18n("Initializing"));
 
-    KConfig conf(configfile);
-    if (conf.accessMode() == KConfig::NoAccess) {
+    QScopedPointer<KConfig> conf;
+    /// TODO KF6: This is fallback logic for an old location for the knsrc files. This should be considered deprecated in KF5,
+    /// and it would make a lot of sense to disable it entirely for KF6
+    bool isRelativeConfig = QFileInfo(configfile).isRelative();
+    QString actualConfig;
+    if (isRelativeConfig) {
+        // Don't do the expensive search unless the config is relative
+        actualConfig = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("knsrcfiles/%1").arg(configfile));
+    }
+    QString configFileName{configfile};
+    if (isRelativeConfig && d->configLocationFallback && actualConfig.isEmpty()) {
+        conf.reset(new KConfig(configfile));
+        qCWarning(KNEWSTUFFCORE) << "Using a deprecated location for the knsrc file" << configfile << " - please contact the author of the software which provides this file to get it updated to use the new location";
+    } else if (isRelativeConfig) {
+        configFileName = QFileInfo(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("knsrcfiles/%1").arg(configfile))).baseName();
+        conf.reset(new KConfig(QString::fromLatin1("knsrcfiles/%1").arg(configfile), KConfig::FullConfig, QStandardPaths::GenericDataLocation));
+    } else {
+        configFileName = QFileInfo(configfile).baseName();
+        conf.reset(new KConfig(configfile));
+    }
+
+    if (conf->accessMode() == KConfig::NoAccess) {
         emit signalErrorCode(KNSCore::ConfigFileError, i18n("Configuration file exists, but cannot be opened: \"%1\"", configfile), configfile);
         qCCritical(KNEWSTUFFCORE) << "The knsrc file '" << configfile << "' was found but could not be opened.";
         return false;
     }
 
     KConfigGroup group;
-    if (conf.hasGroup("KNewStuff3")) {
+    if (conf->hasGroup("KNewStuff3")) {
         qCDebug(KNEWSTUFFCORE) << "Loading KNewStuff3 config: " << configfile;
-        group = conf.group("KNewStuff3");
-    } else if (conf.hasGroup("KNewStuff2")) {
+        group = conf->group("KNewStuff3");
+    } else if (conf->hasGroup("KNewStuff2")) {
         qCDebug(KNEWSTUFFCORE) << "Loading KNewStuff2 config: " << configfile;
-        group = conf.group("KNewStuff2");
+        group = conf->group("KNewStuff2");
     } else {
         emit signalErrorCode(KNSCore::ConfigFileError, i18n("Configuration file is invalid: \"%1\"", configfile), configfile);
         qCCritical(KNEWSTUFFCORE) << configfile << " doesn't contain a KNewStuff3 section.";
         return false;
     }
 
+    d->name = group.readEntry("Name", QString());
     m_categories = group.readEntry("Categories", QStringList());
     m_adoptionCommand = group.readEntry("AdoptionCommand", QString());
 
@@ -132,11 +157,10 @@ bool Engine::init(const QString &configfile)
 
     d->tagFilter = group.readEntry("TagFilter", QStringList());
     if (d->tagFilter.isEmpty()) {
-        d->tagFilter.append(QStringLiteral("ghns_exclude!=1"));
+        d->tagFilter.append(QStringLiteral("ghns_excluded!=1"));
     }
     d->downloadTagFilter = group.readEntry("DownloadTagFilter", QStringList());
 
-    const QString configFileName = QFileInfo(QDir::isAbsolutePath(configfile) ? configfile : QStandardPaths::locate(QStandardPaths::GenericConfigLocation, configfile)).baseName();
     // let installation read install specific config
     if (!m_installation->readConfig(group)) {
         return false;
@@ -145,6 +169,7 @@ bool Engine::init(const QString &configfile)
     connect(m_installation, &Installation::signalEntryChanged, this, &Engine::slotEntryChanged);
 
     m_cache = Cache::getCache(configFileName);
+    qCDebug(KNEWSTUFFCORE) << "Cache is" << m_cache << "for" << configFileName;
     connect(this, &Engine::signalEntryChanged, m_cache.data(), &Cache::registerChangedEntry);
     m_cache->readRegistry();
 
@@ -156,6 +181,11 @@ bool Engine::init(const QString &configfile)
     return true;
 }
 
+QString KNSCore::Engine::name() const
+{
+    return d->name;
+}
+
 QStringList Engine::categories() const
 {
     return m_categories;
@@ -179,6 +209,7 @@ void Engine::loadProviders()
         delete d->m_atticaProviderManager;
         d->m_atticaProviderManager = new Attica::ProviderManager;
         connect(d->m_atticaProviderManager, &Attica::ProviderManager::providerAdded, this, &Engine::atticaProviderLoaded);
+        connect(d->m_atticaProviderManager, &Attica::ProviderManager::failedToLoad, this, &Engine::slotProvidersFailed);
         d->m_atticaProviderManager->loadDefaultProviders();
     } else {
         qCDebug(KNEWSTUFFCORE) << "loading providers from " << m_providerFileUrl;
@@ -289,7 +320,7 @@ void Engine::providerInitialized(Provider *p)
     p->setCachedEntries(m_cache->registryForProvider(p->id()));
     updateStatus();
 
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         if (!p->isInitialized()) {
             return;
         }
@@ -321,7 +352,7 @@ void Engine::reloadEntries()
     m_currentRequest.page = 0;
     m_numDataJobs = 0;
 
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         if (p->isInitialized()) {
             if (m_currentRequest.filter == Provider::Installed) {
                 // when asking for installed entries, never use the cache
@@ -373,6 +404,11 @@ void Engine::setSortMode(Provider::SortMode mode)
     reloadEntries();
 }
 
+Provider::SortMode KNSCore::Engine::sortMode() const
+{
+    return m_currentRequest.sortMode;
+}
+
 void KNSCore::Engine::setFilter(Provider::Filter filter)
 {
     if (m_currentRequest.filter != filter) {
@@ -382,7 +418,12 @@ void KNSCore::Engine::setFilter(Provider::Filter filter)
     reloadEntries();
 }
 
-void KNSCore::Engine::fetchEntryById(const QString& id)
+Provider::Filter KNSCore::Engine::filter() const
+{
+    return m_currentRequest.filter;
+}
+
+void KNSCore::Engine::fetchEntryById(const QString &id)
 {
     m_searchTimer->stop();
     m_currentRequest = KNSCore::Provider::SearchRequest(KNSCore::Provider::Newest, KNSCore::Provider::ExactEntryId, id);
@@ -408,10 +449,15 @@ void Engine::setSearchTerm(const QString &searchString)
     }
 }
 
+QString KNSCore::Engine::searchTerm() const
+{
+    return m_currentRequest.searchTerm;
+}
+
 void Engine::setTagFilter(const QStringList &filter)
 {
     d->tagFilter = filter;
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         p->setTagFilter(d->tagFilter);
     }
 }
@@ -424,7 +470,7 @@ QStringList Engine::tagFilter() const
 void KNSCore::Engine::addTagFilter(const QString &filter)
 {
     d->tagFilter << filter;
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         p->setTagFilter(d->tagFilter);
     }
 }
@@ -432,7 +478,7 @@ void KNSCore::Engine::addTagFilter(const QString &filter)
 void Engine::setDownloadTagFilter(const QStringList &filter)
 {
     d->downloadTagFilter = filter;
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         p->setDownloadTagFilter(d->downloadTagFilter);
     }
 }
@@ -445,7 +491,7 @@ QStringList Engine::downloadTagFilter() const
 void Engine::addDownloadTagFilter(const QString &filter)
 {
     d->downloadTagFilter << filter;
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         p->setDownloadTagFilter(d->downloadTagFilter);
     }
 }
@@ -476,7 +522,7 @@ void Engine::requestData(int page, int pageSize)
 
 void Engine::doRequest()
 {
-    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         if (p->isInitialized()) {
             p->loadEntries(m_currentRequest);
             ++m_numDataJobs;
@@ -529,11 +575,11 @@ void Engine::downloadLinkLoaded(const KNSCore::EntryInternal &entry)
 
 void Engine::uninstall(KNSCore::EntryInternal entry)
 {
-    KNSCore::EntryInternal::List list = m_cache->registryForProvider(entry.providerId());
+    const KNSCore::EntryInternal::List list = m_cache->registryForProvider(entry.providerId());
     //we have to use the cached entry here, not the entry from the provider
     //since that does not contain the list of installed files
     KNSCore::EntryInternal actualEntryForUninstall;
-    foreach (const KNSCore::EntryInternal &eInt, list) {
+    for (const KNSCore::EntryInternal &eInt : list) {
         if (eInt.uniqueId() == entry.uniqueId()) {
             actualEntryForUninstall = eInt;
             break;
@@ -650,7 +696,7 @@ void Engine::updateStatus()
 
 void Engine::checkForUpdates()
 {
-    foreach (QSharedPointer<Provider> p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         Provider::SearchRequest request(KNSCore::Provider::Newest, KNSCore::Provider::Updates);
         p->loadEntries(request);
     }
@@ -658,7 +704,7 @@ void Engine::checkForUpdates()
 
 void KNSCore::Engine::checkForInstalled()
 {
-    foreach (QSharedPointer<Provider> p, m_providers) {
+    for (const QSharedPointer<KNSCore::Provider> &p : qAsConst(m_providers)) {
         Provider::SearchRequest request(KNSCore::Provider::Newest, KNSCore::Provider::Installed);
         request.page = 0;
         request.pageSize = m_pageSize;
@@ -723,3 +769,47 @@ void KNSCore::Engine::setPageSize(int pageSize)
 {
     m_pageSize = pageSize;
 }
+
+QStringList KNSCore::Engine::configSearchLocations(bool includeFallbackLocations)
+{
+    QStringList ret;
+    if(includeFallbackLocations) {
+        ret += QStandardPaths::standardLocations(QStandardPaths::GenericConfigLocation);
+    }
+    QStringList paths = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+    for( const QString& path : paths) {
+        ret << QString::fromLocal8Bit("%1/knsrcfiles").arg(path);
+    }
+    return ret;
+}
+
+void KNSCore::Engine::setConfigLocationFallback(bool enableFallback)
+{
+    d->configLocationFallback = enableFallback;
+}
+
+QSharedPointer<KNSCore::Provider> KNSCore::Engine::provider(const QString &providerId) const
+{
+    return m_providers.value(providerId);
+}
+
+QSharedPointer<KNSCore::Provider> KNSCore::Engine::defaultProvider() const
+{
+    if (m_providers.count() > 0)
+        return m_providers.constBegin().value();
+    return nullptr;
+}
+
+KNSCore::CommentsModel *KNSCore::Engine::commentsForEntry(const KNSCore::EntryInternal &entry)
+{
+    CommentsModel *model = d->commentsModels[entry];
+    if (!model) {
+        model = new CommentsModel(this);
+        model->setEntry(entry);
+        connect(model, &QObject::destroyed, this, [=](){
+            d->commentsModels.remove(entry);
+        });
+        d->commentsModels[entry] = model;
+    }
+    return model;
+}
